{$IFNDEF MultiMediaTimerPas}
{$DEFINE MultiMediaTimerPas}

{$IFNDEF WIN32}
  {$DEFINE WIN32}
{$ENDIF WIN32}

unit MultiMediaTimer;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Mmsystem;

type
  TMMProc = procedure of object;
  TMultiMediaTimer = class(TObject)
  const
    ST_MMTIMER_INIT      = 0;
    ST_MMTIMER_STANDBY   = 1;
    ST_MMTIMER_EXECUTING = 2;
    ST_MMTIMER_STOP      = 3;
    DEFAULT_MMTIMER_INTERVAL  = 10;
    MMTIMER_TARGET_RESOLUTION	= 1;  {* 1millisecond target resolution *}
  private
    m_nStatus: integer;   {* Status *}
    m_wTimerRes: word;    {* Timer Resolution *}
    m_flgCallback: bool;  {* Flag: Callback Executing *}
    m_hMMTimer: MMRESULT; {* Handle of Multimedia Tiemr *}
    m_pfncMMProc: TMMProc; {* Handle of callback *}
    function Max(nValue1, nValue2: integer):integer;
    function Min(nValue1, nValue2: integer):integer;
  public
    constructor Create;
    destructor Destroy; override;
    function StartTimer(nInterval: integer; CallBack: TMMProc):Bool;
    function StopTimer():bool;
    function IsTimerProcExecuting():bool;
    procedure SetTimerProcExecuting();
    procedure ClearTimerProcExecuting();
    function IsTimerFuncAvailable():bool;
    procedure ExecuteTimerProc();
  end;

procedure gMMTimerProc(TimerID, Msg: UINT; dwUser, dw1, dw2: DWORD_PTR); stdcall;

implementation

{*
 /brief Constructor
*}
constructor TMultiMediaTimer.Create;
begin
  m_nStatus     := ST_MMTIMER_INIT;
  m_wTimerRes   := MMTIMER_TARGET_RESOLUTION;
  m_flgCallback := FALSE;
  m_hMMTimer    := 0;
  m_pfncMMProc  := nil;
end;

{*
 /brief Destructor
*}
Destructor TMultiMediaTimer.Destroy;
begin
  StopTimer();
end;

{*
 /param [in] nValue1  Value 1
 /param [in] nValue2  Value 2
 /return maximum value.
 /brief Compare value 1 and value 2
*}
function TMultiMediaTimer.Max(nValue1, nValue2: integer):integer;
begin
 if (nValue1 > nValue2) then
  Result := nValue1
 else
  Result := nValue2;
end;

{*
 /param [in] nValue1  Value 1
 /param [in] nValue2  Value 2
 /return minimum value.
 /brief Compare value 1 and value 2
*}
function TMultiMediaTimer.Min(nValue1, nValue2: integer):integer;
begin
 if (nValue1 > nValue2) then
  Result := nValue2
 else
  Result := nValue1;
end;

{*
 /param [in] nInterval  Timer interval value
 /param [in] CallBack   Handle of cyclic function
 /return N/A.
 /brief Start Multi Media Tiemr
*}
function TMultiMediaTimer.StartTimer(nInterval: integer; CallBack: TMMProc):bool;
 var
  tTimeCaps: TIMECAPS;
  nMMRslt: MMRESULT;
begin
 if (m_nStatus = ST_MMTIMER_INIT) then
  begin
   if (timeGetDevCaps(@tTimeCaps, sizeof(TIMECAPS)) = TIMERR_NOERROR) then
    begin
     m_wTimerRes := Min(Max(tTimeCaps.wPeriodMin, MMTIMER_TARGET_RESOLUTION), tTimeCaps.wPeriodMax);
     timeBeginPeriod(m_wTimerRes);
     m_nStatus := ST_MMTIMER_STANDBY;
     Result := true;
    end
   else
    Result := false;
  end;
 if (m_nStatus = ST_MMTIMER_STANDBY) then
  begin
   m_pfncMMProc := CallBack;
   nMMRslt := timeSetEvent(nInterval, m_wTimerRes, gMMTimerProc, LongWord(Self), TIME_PERIODIC);
   if (nMMRslt = 0) then
    begin
     m_pfncMMProc := nil;
     m_hMMTimer := 0;
     StopTimer();
     Result := false;
    end
   else
    begin
     m_hMMTimer := nMMRslt;
     m_nStatus := ST_MMTIMER_EXECUTING;
     Result := true;
    end;
  end
 else
  Result := true;
end;

{*
 /return N/A.
 /brief Stop Multi Media Tiemr
*}
function TMultiMediaTimer.StopTimer():bool;
begin
 if ( m_nStatus > ST_MMTIMER_STANDBY ) then
  begin
   if (m_hMMTimer <> 0) then
    begin
     timeKillEvent(m_hMMTimer);
     m_pfncMMProc := nil;
     m_hMMTimer := 0;
     m_flgCallback := false;
     m_nStatus := ST_MMTIMER_STANDBY;
    end;
  end;
 if (m_nStatus = ST_MMTIMER_STANDBY) then
  begin
   timeEndPeriod(m_wTimerRes);
   m_wTimerRes := MMTIMER_TARGET_RESOLUTION;
   m_nStatus := ST_MMTIMER_INIT;
  end;
  Result := true;
end;

{*
 /return status.
 /brief Check Timer Procedure Executing
*}
function TMultiMediaTimer.IsTimerProcExecuting():bool;
begin
 Result := m_flgCallback;
end;

{*
 /return N/A.
 /brief Set Timer Procedure Executing
*}
procedure TMultiMediaTimer.SetTimerProcExecuting();
begin
 m_flgCallback := true;
end;

{*
 /return N/A.
 /brief Set Timer Procedure Watting
*}
procedure TMultiMediaTimer.ClearTimerProcExecuting();
begin
 m_flgCallback := false
end;

{*
 /return status.
 /brief Check Timer Procedure Available
*}
function TMultiMediaTimer.IsTimerFuncAvailable():bool;
begin
 if (Assigned(m_pfncMMProc)) then
  begin
   Result := true;
  end
 else
  begin
   Result := false;
  end;
end;

{*
 /return N/A.
 /brief Call Timer Procedure
*}
procedure TMultiMediaTimer.ExecuteTimerProc();
var
 proc: TMMProc;
begin
 proc := TMMProc(m_pfncMMProc);
 proc();
end;

procedure gMMTimerProc(TimerID, Msg: UINT; dwUser, dw1, dw2: DWORD_PTR); stdcall;
 var
  MMTimer: TMultiMediaTimer;
 begin
  MMTimer := TMultiMediaTimer(dwUser);
  if Assigned(MMTimer) then
   begin
    if (MMTimer.IsTimerProcExecuting() = false) then
     begin
      if (MMTimer.IsTimerFuncAvailable()) then
       begin
        MMTimer.SetTimerProcExecuting();
        MMTimer.ExecuteTimerProc();
        MMTimer.ClearTimerProcExecuting();
       end;
     end;
   end;
 end;

{$ENDIF MultiMediaTimerPas}
end.




