unit Unit_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.StdCtrls,
  Vcl.ExtCtrls, VCLTee.TeEngine, VCLTee.Series, VCLTee.TeeProcs, VCLTee.Chart, MultiMediaTimer;

type
  TForm_Main = class(TForm)
    Chart_Main: TChart;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    Series3: TFastLineSeries;
    Series4: TFastLineSeries;
    Panel_Main: TPanel;
    Button_Start: TButton;
    Button_Stop: TButton;
    Button_Close: TButton;
    procedure FormShow(Sender: TObject);
    procedure Button_StartClick(Sender: TObject);
    procedure Button_StopClick(Sender: TObject);
    procedure Button_CloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  const
    MAX_SERIES = 4;
  private
    { Private }
    m_tMMTimerMain: TMultiMediaTimer;
  public
    { Public }
    m_uint32Count: LongWord;
    m_rl64StartTime: double;
    m_arl64Value: array [0..MAX_SERIES] of double;
    procedure CycleEvent();
    procedure SeriesAfterAdd(Sender: TChartSeries; ValueIndex: integer);
  end;

var
  Form_Main: TForm_Main;

implementation

{*
 Form Show
*}
procedure TForm_Main.FormShow(Sender: TObject);
begin
 try
  m_tMMTimerMain := TMultiMediaTimer.Create();
 except
  ShowMessage('Allocation fault');
  Close();
 end;
end;

{*
 Form Show
*}
procedure TForm_Main.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(m_tMMTimerMain) then
   m_tMMTimerMain.Destroy;
end;

{*
 Start
*}
procedure TForm_Main.Button_StartClick(Sender: TObject);
var
  I: Integer;
  bRslt : Bool;
begin
 m_rl64StartTime := Now();
 for I := 0 to (MAX_SERIES -1) do
  begin
    m_arl64Value[I] := I * 100.0;
  end;
  m_uint32Count := 0;
  Chart_Main.BottomAxis.Automatic := true;
  for I := 0 to (Chart_Main.SeriesCount() - 1) do
    begin
      Chart_Main.Series[I].Clear();
      Chart_Main.Series[I].OnAfterAdd := Self.SeriesAfterAdd;
    end;
  bRslt := m_tMMTimerMain.StartTimer(2, CycleEvent);
  if bRslt = false then
   ShowMessage('Start Error');
end;

{*
 Stop
*}
procedure TForm_Main.Button_StopClick(Sender: TObject);
begin
 m_tMMTimerMain.StopTimer();
 Chart_Main.BottomAxis.SetMinMax(Chart_Main.Series[0].XValues.MinValue, Chart_Main.Series[0].XValues.MaxValue);
end;

{*
 Close
*}
procedure TForm_Main.Button_CloseClick(Sender: TObject);
begin
 Close();
end;

{*
 Cycle Event
*}
procedure TForm_Main.CycleEvent();
var
 rl64Time: double;
  I: Integer;
begin
 rl64Time := (Now() - m_rl64StartTime) * 86400.0;
 m_uint32Count := (m_uint32Count + 1);
 if (m_uint32Count = 500) then
  begin
   m_uint32Count := 0;
  end;
 for I := 0 to (MAX_SERIES - 1) do
  begin
   if (m_uint32Count < 250) then
    begin
     m_arl64Value[I] := m_arl64Value[I] + 1.0;
    end
   else
    begin
     m_arl64Value[I] := m_arl64Value[I] - 1.0;
    end;
   Chart_Main.Series[I].AddXY(rl64Time, m_arl64Value[I], '', Chart_Main.Series[I].SeriesColor);
  end;
end;

{*
*}
procedure TForm_Main.SeriesAfterAdd(Sender: TChartSeries; ValueIndex: integer);
var
 rlVal: double;
begin
 rlVal := Sender.XValues.MaxValue;
 Sender.GetHorizAxis.SetMinMax(rlVal - 10.0, rlVal);
end;


{$R *.dfm}

end.
